#!/bin/bash

RED='\033[0;31m'
YELLOW='\033[1;33m'
GREEN='\033[0;32m'
LIGHT_BLUE='\033[1;34m'
NC='\033[0m'

config_file='config.json'

printf "\n%b**********************************************%b\n" "$LIGHT_BLUE" "$NC"
printf "%b* Welcome to the Server-Utils Script by VS-Shop *%b\n" "$GREEN" "$NC"
printf "%b************************************************%b\n\n" "$LIGHT_BLUE" "$NC"
printf "%bLoading configurations and preparing your system report...%b\n\n" "$YELLOW" "$NC"

if [ ! -f "$config_file" ]; then
    printf "%bError: Configuration file %s is missing.%b\n" "$RED" "$config_file" "$NC"
    exit 1
else
    printf "%bConfiguration file detected successfully.%b\n" "$GREEN" "$NC"
fi

logo_path=$(jq -r '.logo_path' "$config_file")
email_header=$(jq -r '.email_header' "$config_file")
recipient=$(jq -r '.recipient' "$config_file")
email_from=$(jq -r '.email_from' "$config_file")

# Function to check for errors, especially logo presence
check_errors() {
    if [ ! -f "$logo_path" ]; then
        printf "%bWarning: Logo file is missing at %s.%b\n" "$YELLOW" "$logo_path" "$NC"
    else
        printf "%bLogo file detected successfully at %s.%b\n" "$GREEN" "$logo_path" "$NC"
    fi
}

get_performance_metrics() {
    cpu_load=$(uptime | awk -F'load average: ' '{ print $2 }' | cut -d',' -f1)
    ram_usage=$(free -m | awk '/Mem:/ {printf "%.2f%%", $3/$2 * 100}')
    disk_usage=$(df -h / | awk 'NR==2{print $5}')

    read si so <<< $(vmstat 1 2 | tail -1 | awk '{print $7, $8}')

    metrics_report="<ul>"
    metrics_report+="<li>CPU Load Average: $cpu_load</li>"
    metrics_report+="<li>RAM Usage: $ram_usage</li>"
    metrics_report+="<li>Disk Usage: $disk_usage</li>"
    metrics_report+="<li>Memory Swap-in: $si, Memory Swap-out: $so</li>"
    metrics_report+="</ul>"

    printf "\n%bSystem Performance Metrics:%b\n" "$LIGHT_BLUE" "$NC"
    echo "$metrics_report" | sed 's/<ul>/\n/g; s/<\/li>/\n/g; s/<li>/  - /g; s/<\/ul>//g; s/<[^>]*>//g;'

    echo "$metrics_report" # Return the metrics report for email
}

send_email() {
    local hostname=$(hostname)
    local subject="$email_header for $hostname"
    local body="<h2>$email_header for $hostname</h2><br>$(get_performance_metrics)"
    local boundary="boundary_string"

    {
        echo "To: $recipient"
        echo "From: $email_from"
        echo "Subject: $subject"
        echo "MIME-Version: 1.0"
        echo "Content-Type: multipart/related; boundary=\"$boundary\""

        echo "--$boundary"
        echo "Content-Type: text/html; charset=\"UTF-8\""
        echo "Content-Transfer-Encoding: 7bit"
        echo ""
        echo "$body"
        echo ""

        if [ -f "$logo_path" ]; then
            echo "--$boundary"
            echo "Content-Type: image/png"
            echo "Content-Transfer-Encoding: base64"
            echo "Content-ID: <logo.png>"
            base64 "$logo_path"
            echo ""
        fi

        echo "--$boundary--"
    } | sendmail -t

    echo "Email sent to: $recipient" >> email_log.txt
    echo "Subject: $subject" >> email_log.txt
    echo "Body: $body" >> email_log.txt
    echo "-------------------------" >> email_log.txt
}

main() {
    check_errors
    read -p "No errors found. Would you like to proceed? (yes/no): " response
    if [[ $response == "yes" ]]; then
        printf "%bPlease note: The script may take up to 5 minutes to run after you proceed.%b\n" "$YELLOW" "$NC"
        # Display metrics in terminal before sending email
        get_performance_metrics
        send_email
        printf "%bSuccess! The system report has been sent.%b\n" "$GREEN" "$NC"
    else
        printf "%bScript aborted by the user.%b\n" "$RED" "$NC"
    fi
}

main
